from textual.app import App, ComposeResult
from textual.binding import Binding
from textual.widgets import Static, Button, Footer
from textual.containers import Container, Vertical
from textual.screen import Screen
from textual import events
from textual.pilot import Pilot

from esse3_student.primitives import ExamName
from esse3_student.tui.booklet import Booklet
from esse3_student.tui.exams import Exams
from esse3_student.tui.reservations import Reservations
from esse3_student.tui.taxes import Taxes


class Header(Static):
    pass


class HomePage(Screen):

    def compose(self) -> ComposeResult:
        yield Header("Homepage", classes="header")
        yield Container(
            Static("[bold][green]Esse3 command line utility[/green][/bold] :computer:", classes="title"),
            Vertical(
                Button("[bold]Booklet[/] - [italic]show all activities[/]", id="booklet"),
                Button("[bold]Taxes[/] - [italic]show all taxes[/]", id="taxes"),
                Button("[bold]Exams[/] - [italic]show available exams[/]", id="exams"),
                Button("[bold]Reservations[/] - [italic]show booked exams[/]", id="reservations"),
            ),
            id="homepage"
        )
        yield Footer()


class Tui(App):

    CSS_PATH = "style.css"

    SCREENS = {"homepage": HomePage()}

    BINDINGS = [
        Binding(key="escape", action="key_escape", description="exit"),
    ]

    def on_mount(self) -> None:
        self.push_screen("homepage")

    def modify(self, action: str) -> None:
        selected_exam_name = None
        at_least_one_checkbox_selected = False
        selected_exams = []
        for checkbox in self.query("Checkbox"):
            if checkbox.value:
                selected_exam_name = checkbox.name_value
                selected_exams.append(ExamName(selected_exam_name))
                at_least_one_checkbox_selected = True
        if not at_least_one_checkbox_selected:
            return

        name_value = f"{action}-{selected_exam_name}"

        if action == "add":
            if not self.is_screen_installed(name_value):
                self.install_screen(Exams.Add(selected_exams), name=name_value)
        elif action == "remove":
            if not self.is_screen_installed(name_value):
                self.install_screen(Reservations.Remove(selected_exams), name=name_value)

        self.push_screen(name_value)

        for c in self.query("Checkbox"):
            c.value = False

        if action == "add":
            if self.is_screen_installed("reservations"):
                self.uninstall_screen("reservations")
            if self.is_screen_installed("remove-" + selected_exam_name):
                self.uninstall_screen("remove-" + selected_exam_name)
        else:
            if self.is_screen_installed("exams"):
                self.uninstall_screen("exams")
            if self.is_screen_installed("add-" + selected_exam_name):
                self.uninstall_screen("add-" + selected_exam_name)

    def on_button_pressed(self, event: Button.Pressed) -> None:
        screens = {"booklet": Booklet(), "reservations": Reservations(), "taxes": Taxes(), "exams": Exams()}
        changes = ["add", "remove"]
        if event.button.id in changes:
            self.modify(event.button.id)

        elif event.button.id in screens:
            if not self.is_screen_installed(f"{event.button.id}"):
                self.install_screen(screens[f"{event.button.id}"], name=f"{event.button.id}")
                self.push_screen(f"{event.button.id}")
            else:
                self.push_screen(f"{event.button.id}")

    async def on_key(self, event: events.Key):
        if event.key == "up":
            pilot = Pilot(self)
            await pilot.press("shift+tab")
        if event.key == "down":
            pilot = Pilot(self)
            await pilot.press("tab")

    def action_key_escape(self) -> None:
        self.exit()

    def action_return(self, screen):
        screens = {"booklet": Booklet(), "reservations": Reservations(), "taxes": Taxes(), "exams": Exams()}
        self.uninstall_screen(self.pop_screen())
        self.pop_screen()
        self.uninstall_screen(screen)
        self.install_screen(screens[f"{screen}"], name=f"{screen}")
        self.push_screen(screen)

    def action_homepage(self, screen):
        self.pop_screen()
        self.pop_screen()
        self.uninstall_screen(screen)



import dataclasses
import time
import subprocess

import screeninfo
from dataclasses import InitVar
from typing import List, Tuple, Union, Any

import typeguard
from selenium import webdriver
from selenium.common import WebDriverException
from selenium.common.exceptions import NoSuchElementException
from selenium.common.exceptions import TimeoutException
from selenium.webdriver import Keys
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

from esse3_student.utils import validators
from esse3_student.primitives import Username, Password, ExamName, Description, Date, SigningUp, \
    AcademicYear, Cfu, ExamStatus, BookletGrade, TaxeID, Amount, TaxeStatus

ESSE3_SERVER = "https://unical.esse3.cineca.it"
LOGIN_URL = f'{ESSE3_SERVER}/auth/Logon.do?menu_opened_cod='
LOGOUT_URL = f'{ESSE3_SERVER}/Logout.do?menu_opened_cod='
EXAMS_URL = f'{ESSE3_SERVER}/auth/studente/Appelli/AppelliF.do?menu_opened_cod=menu_link-navbox_studenti_Esami'
RESERVATIONS_URL = f'{ESSE3_SERVER}/auth/studente/Appelli/BachecaPrenotazioni.do?menu_opened_cod=menu_link-navbox_studenti_Esami'
BOOKLET_URL = f'{ESSE3_SERVER}/auth/studente/Libretto/LibrettoHome.do?menu_opened_cod=menu_link-navbox_studenti_Carriera'
TAXES_URL = f'{ESSE3_SERVER}/auth/studente/Tasse/ListaFatture.do?menu_opened_cod=menu_link-navbox_studenti_Segreteria'


def change_esse3_server(url):
    global ESSE3_SERVER, LOGIN_URL, LOGOUT_URL, EXAMS_URL, RESERVATIONS_URL, BOOKLET_URL, TAXES_URL

    LOGIN_URL = LOGIN_URL.replace(ESSE3_SERVER, url, 1)
    LOGOUT_URL = LOGOUT_URL.replace(ESSE3_SERVER, url, 1)
    EXAMS_URL = EXAMS_URL.replace(ESSE3_SERVER, url, 1)
    RESERVATIONS_URL = RESERVATIONS_URL.replace(ESSE3_SERVER, url, 1)
    BOOKLET_URL = BOOKLET_URL.replace(ESSE3_SERVER, url, 1)
    TAXES_URL = TAXES_URL.replace(ESSE3_SERVER, url, 1)

    ESSE3_SERVER = url


def prompt_position() -> None:

    result = subprocess.run(["xdotool", "getactivewindow"], capture_output=True)
    window_id = result.stdout.decode().strip()
    screen_height = subprocess.check_output(["xdotool", "getdisplaygeometry"]).decode().split()[1]

    subprocess.run(["xdotool", "windowmove", window_id, "0", str(int(screen_height) // 4)])
    subprocess.run(["xdotool", "windowsize", window_id, "50%", "-1"])
    # con -1 l'altezza rimane invariata altrimenti lasciare  str(int(screen_height) // 2)


def driver_position(driver) -> None:

    #self.driver.maximize_window()
    screen = screeninfo.get_monitors()[0]
    width, height = screen.width, screen.height
    driver.set_window_size(width // 2, height)
    driver.set_window_position(width // 2, 0)


@typeguard.typechecked
@dataclasses.dataclass(frozen=True)
class Esse3Wrapper:
    key: InitVar[object]
    username: InitVar[Username]
    password: InitVar[Password]
    debug: bool = dataclasses.field(default=False)
    driver: webdriver.Chrome = dataclasses.field(default_factory=webdriver.Chrome)
    __key = object()

    def __post_init__(self, key: object, username: Username, password: Password):
        validators.validate_dataclass(self)
        validators.validate('key', key, equals=self.__key, help_msg="Can only be instantiated using a factory method")

        self.__login(username, password)

    def __del__(self):
        if not self.debug:
            try:
                self.__logout()
                self.driver.close()
            except WebDriverException:
                pass
            except ValueError:
                pass

    @classmethod
    def create(cls, username: str, password: str, debug: bool = False, detached: bool = False,
               headless: bool = True) -> 'Esse3Wrapper':
        options = webdriver.ChromeOptions()
        options.headless = headless
        if debug or detached:
            options.add_experimental_option("detach", True)
            prompt_position()
        driver = webdriver.Chrome(options=options)
        driver_position(driver)

        return Esse3Wrapper(
            key=cls.__key,
            username=Username.parse(username),
            password=Password.parse(password),
            debug=debug,
            driver=driver,
        )

    @property
    def is_headless(self) -> bool:
        return self.driver.execute_script("return navigator.webdriver")

    def __login(self, username: Username, password: Password) -> None:

        self.driver.get(LOGIN_URL)

        """try:
            WebDriverWait(self.driver, 2).until \
                (EC.visibility_of_element_located(
                    (By.XPATH, "//*[@id='c-s-bn']")))
            self.driver.find_element(By.XPATH, "//*[@id='c-s-bn']").click()
        except:
            pass"""

        try:
            self.driver.find_element(By.ID, 'u').send_keys(username.value)
            self.driver.find_element(By.ID, 'p').send_keys(password.value)
            self.driver.find_element(By.ID, 'btnLogin').send_keys(Keys.RETURN)
            carrier = WebDriverWait(self.driver, 2).until(EC.visibility_of_element_located(
                (By.XPATH, "/html/body/div[2]/div/div/main/div[3]/div/div/table/tbody/tr[1]/td[5]/div/a")))
            carrier.click()
        except Exception:
            raise RuntimeError("Wrong credentials")

    def __logout(self) -> None:
        self.driver.get(LOGOUT_URL)

    def minimize(self) -> None:
        self.driver.minimize_window()

    def fetch_exams(self) -> list[tuple[ExamName, Date, SigningUp, Description]]:

        self.driver.get(EXAMS_URL)
        try:
            exams = WebDriverWait(self.driver, 5).until(
                EC.visibility_of_all_elements_located((By.XPATH,
                        "//*[@id='app-tabella_appelli']/tbody/tr")))
        except TimeoutException:
            return []

        rows = []

        for exam in exams:
            elements = exam.find_elements(By.TAG_NAME, "td")
            name = ExamName.of(elements[1].text)
            date = Date.of(elements[2].text)
            signing_up = SigningUp.of(elements[3].get_attribute('innerText').replace("\n", " - "))
            description = Description.of(elements[4].get_attribute("innerHTML"))

            rows.append((name, date, signing_up, description))

        return rows

    def fetch_reservations(self) -> list:

        self.driver.get(RESERVATIONS_URL)
        try:
            reservations = WebDriverWait(self.driver, 5).until(
                EC.visibility_of_all_elements_located((By.XPATH, "//*[@id='boxPrenotazione']")))
        except:
            return []

        rows = []

        index = 2
        for reservation in reservations:
            name = reservation.find_element(By.XPATH,
                                            f"/html/body/div[2]/div/div/main/div[3]/div/div/div/div[{index}]/h2").text
            start = name.find(" [")
            name = name[:start]
            dict = {"Name": name}
            elements = reservation.find_elements(By.XPATH, "./dl/dt")
            for position, element in enumerate(elements, start=1):
                key = element.text
                value = element.find_element(By.XPATH, f"../dd[{position}]").text
                if position == 1:
                    dict["Date"] = key
                elif key != "Riservato per" and key != "Data Prenotazione" and key != '':
                    dict[key] = value
            rows.append(dict)
            index += 2

        return rows

    def add(self, names: list[ExamName]) -> tuple[list[ExamName], int]:

        self.driver.get(EXAMS_URL)
        click = 7
        try:
            exams = WebDriverWait(self.driver, 5).until(
                EC.visibility_of_all_elements_located((By.XPATH, "//table/tbody/tr")))
        except:
            return [], click

        added = []

        while names:
            name = names.pop().value
            wrong = True
            for i, exam in enumerate(exams, start=1):
                exam_name = name.upper()
                if exam.find_element(By.XPATH, f"//table/tbody/tr[{i}]/td[2]").text == exam_name:
                    added.append(ExamName.of(f"{exam_name}"))
                    wrong = False
                    exam_link = self.driver.find_element(By.XPATH, f"//table/tbody/tr[{i}]/td/div/a")
                    self.driver.execute_script("arguments[0].scrollIntoView();", exam_link)
                    exam_link.send_keys(Keys.ENTER)
                    save_button = self.driver.find_element(By.XPATH, "//*[@id='btnSalva']")
                    self.driver.execute_script("arguments[0].scrollIntoView();", save_button)
                    save_button.send_keys(Keys.ENTER)
                    click += 2
                    break

            if len(names) != 0:
                self.driver.get(EXAMS_URL)
                if not wrong:
                    click += 1
                exams = WebDriverWait(self.driver, 10).until(
                    EC.visibility_of_all_elements_located((By.XPATH, "//table/tbody/tr")))

        return added, click

    def remove(self, names: list[ExamName]) -> tuple[{}, int]:

        self.driver.get(RESERVATIONS_URL)
        click = 7

        try:
            box_prenotazione = WebDriverWait(self.driver, 5).until(
                EC.visibility_of_all_elements_located((By.XPATH, "//*[@id='boxPrenotazione']")))
            tool_bar = WebDriverWait(self.driver, 5).until(
                EC.visibility_of_all_elements_located((By.XPATH, "//*[@id='toolbarAzioni']")))
        except:
            return {}, click

        values = {
            0: [],
            1: [],
        }

        for reservation in names:
            found = False
            for i, name in enumerate(box_prenotazione, start=1):
                value = name.find_element(By.CLASS_NAME, "record-h2").text.split(" [")[0].strip()
                reservation_name = reservation.value.upper()
                if value == reservation_name:
                    try:
                        element = tool_bar[i - 1].find_element(By.ID, 'btnCancella')
                        element.click()
                        confirm = self.driver.find_element(By.XPATH, "//*[@id='btnConferma']")
                        confirm.click()
                        click += 2
                        values[1].append(reservation_name)
                        found = True
                        break
                    except NoSuchElementException:
                        pass
            if not found:
                values[0].append(reservation.value)
            else:
                self.driver.get(RESERVATIONS_URL)
                WebDriverWait(self.driver, 10).until(
                    EC.visibility_of_element_located((By.XPATH, "//*[@id='textHeader']")))
                box_prenotazione = self.driver.find_elements(By.XPATH, "//*[@id='boxPrenotazione']")
                tool_bar = self.driver.find_elements(By.XPATH, "//*[@id='toolbarAzioni']")

        values = {k: v for k, v in values.items() if v}

        return values, click

    def fetch_booklet(self) -> tuple[list[tuple[ExamName, AcademicYear, Cfu, ExamStatus, BookletGrade, Date]], tuple[float, int]]:

        self.driver.get(BOOKLET_URL)

        exams = WebDriverWait(self.driver, 10).until(
            EC.visibility_of_all_elements_located((By.XPATH, "/html/body/div[2]/div/div/main/div[3]/div/div/div/table/tbody/tr")))

        weighted_average = self.driver.find_element(By.XPATH, "//div[@id='boxMedie']//li[2]").text.split()[4]
        cfu_achieved = 0

        rows = []

        for i, exam in enumerate(exams, start=1):

            name = exam.find_element(By.XPATH, f"/html/body/div[2]/div/div/main/div[3]/div/div/div/table/tbody/tr[{i}]/td[1]/a").text
            academic_year = exam.find_element(By.XPATH, f"//*[@id='tableLibretto']/tbody/tr[{i}]/td[2]").get_attribute("innerHTML")
            cfu = exam.find_element(By.XPATH, f"//*[@id='tableLibretto']/tbody/tr[{i}]/td[3]").get_attribute("innerHTML")
            status = exam.find_element(By.XPATH, f"//*[@id='tableLibretto']/tbody/tr[{i}]/td[4]/img").get_attribute('aria-label')
            grade_date = exam.find_element(By.XPATH, f"//*[@id='tableLibretto']/tbody/tr[{i}]/td[6]").get_attribute("innerHTML")

            # Remove characters preceding the first space immediately after the hyphen.
            name = name[name.index(" - ") + 3:]
            # Separate the string "grade_date" into two parts using the separator " - "
            grade, date = (grade_date.replace("&nbsp;-&nbsp;", " - ").split(" - ") + [''])[:2]

            grade = "ELIGIBLE" if grade == "IDO" else grade or "..."
            date = Date.of(date or "None")

            if status == "Superata":
                status = "Passed"
                cfu_achieved += int(cfu)
            else:
                status = "Ex officio assigned frequency"

            rows.append((ExamName.of(name), AcademicYear.of(int(academic_year)), Cfu.of(cfu),
                         ExamStatus.of(status), BookletGrade.of(grade), date))

        statistics = (float(weighted_average), int(cfu_achieved))

        return rows, statistics


    def fetch_taxes(self) -> tuple[list[tuple[TaxeID, Date, Amount, TaxeStatus]], int]:

        self.driver.get(TAXES_URL)
        click = 7

        taxes = WebDriverWait(self.driver, 10).until(
            EC.visibility_of_all_elements_located((By.XPATH, "//*[@id='tasse-tableFatt']/tfoot/tr/td/div/ul/li/a")))
        rows = []

        start = 3
        while start < len(taxes):
            page = self.driver.find_element(By.XPATH, f"//*[@id='tasse-tableFatt']/tfoot/tr/td/div/ul/li[{start}]/a")
            if 0 < int(page.text) < 10:
                if start != 3:
                    page.click()
                    click += 1
                time.sleep(1)
                taxes = self.driver.find_elements(By.XPATH, "/html/body/div[2]/div/div/main/div[3]/div/div/table[1]/tbody/tr")
                for index, taxe in enumerate(taxes, start=1):

                    id = taxe.find_element(By.XPATH, f"/html/body/div[2]/div/div/main/div[3]/div/div/table[1]/tbody/tr[{index}]/td[1]/a").text
                    expiration_date = taxe.find_element(By.XPATH, f"/html/body/div[2]/div/div/main/div[3]/div/div/table[1]/tbody/tr[{index}]/td[4]").get_attribute('data-sort-value')
                    amount = taxe.find_element(By.XPATH, f"/html/body/div[2]/div/div/main/div[3]/div/div/table[1]/tbody/tr[{index}]/td[5]").text
                    payment_status = taxe.find_element(By.XPATH, f"/html/body/div[2]/div/div/main/div[3]/div/div/table[1]/tbody/tr[{index}]/td[6]").text

                    if expiration_date is not None:
                        year, month, day = expiration_date.split("-")
                        expiration_date = f"{day}/{month}/{year}"

                    rows.append((TaxeID.of(id), Date.of(expiration_date or "None"), Amount.of(amount), TaxeStatus.of(payment_status)))
            start = start + 1

        return rows, click



import datetime

from esse3_student.utils.primitives import bounded_string, bounded_integer


@bounded_string(min_length=3, max_length=30, pattern=r'[A-Za-z0-9]*')
class Username:
    pass


@bounded_string(min_length=5, max_length=100, pattern=r'[^\n]*')
class Password:
    pass


@bounded_string(min_length=3, max_length=50, pattern=r"[A-Z a-z ’']+")
class ExamName:
    pass


@bounded_string(min_length=3, max_length=255, pattern=r"[-A-Z a-z 0-9 \.\/:[\]’'&\n]+")
class Description:
    pass


@bounded_string(min_length=4, max_length=10, pattern=r"^(?:(?:0?[1-9]|[12]\d|3[01])\/(?:0?[1-9]|1[0-2])\/(?:19|20)\d{2}|None)$")
class Date:
    pass


@bounded_string(min_length=23, max_length=23, pattern=r"^(?:(?:0?[1-9]|[12]\d|3[01])\/(?:0?[1-9]|1[0-2])\/(?:19|20)\d{2})\s*-\s*(?:(?:0?[1-9]|[12]\d|3[01])\/(?:0?[1-9]|1[0-2])\/(?:19|20)\d{2})$")
class SigningUp:
    pass


@bounded_string(min_length=7, max_length=7, pattern=r"[0-9]+")
class TaxeID:
    pass


@bounded_string(min_length=7, max_length=18, pattern=r"( pagato| non pagato| pagato confermato)")
class TaxeStatus:
    pass


@bounded_string(min_length=6, max_length=9, pattern=r"^-?\d{1,3}(?:\.\d{3})*(?:,\d{2})?\s€$")
class Amount:
    pass


@bounded_integer(min_value=1, max_value=3)
class AcademicYear:
    pass


@bounded_integer(min_value=2010, max_value=datetime.datetime.now().year)
class Year:
    pass


@bounded_string(min_length=6, max_length=29, pattern=r"(Passed|Ex officio assigned frequency)")
class ExamStatus:
    pass


@bounded_integer(min_value=18, max_value=30)
class Grade:
    pass


@bounded_string(min_length=2, max_length=8, pattern=r"(ELIGIBLE|1[89]|[2-2][0-9]|30|...)")
class BookletGrade:
    pass


@bounded_string(min_length=1, max_length=2, pattern=r'(3|6|9|12|24)')
class Cfu:
    pass





import asyncio

from esse3_student import cli
from esse3_student.primitives import ExamName, Date, SigningUp, Description

from textual.app import ComposeResult
from textual.binding import Binding
from textual.widgets import Static, Button, Footer, Checkbox
from textual.containers import Container, Horizontal, Vertical
from textual.screen import Screen

from rich import box
from rich.table import Table


class Header(Static):
    pass


class Exams(Screen):

    class Tables(Static):

        def __init__(self, exams: list) -> None:
            self.exams = exams
            super().__init__()

        def on_mount(self) -> None:
            self.get_table(self.exams)

        def get_table(self, exams: list) -> None:

            table = Table(box=box.SIMPLE_HEAD, style="rgb(139,69,19)")
            table.add_column("#", justify="center", style="bold red")
            table.add_column("Name", justify="center", style="bold cyan")
            table.add_column("Date", justify="center", style="bold green")
            table.add_column("Signing up", justify="center", style="bold yellow")
            table.add_column("Description", justify="center", style="bold #f7ecb5")

            for index, (name, date, signing_up, description) in enumerate(exams, start=1):
                table.add_row(str(index), name.value, date.value, signing_up.value, description.value)

            self.update(table)

    class Line(Horizontal):

        def __init__(self, exam) -> None:
            self.value = exam
            super().__init__()

        def on_mount(self) -> None:
            self.mount(self.Name(self.value[0].value))
            self.mount(self.Check(self.value[0].value))

        class Name(Static):
            def __init__(self, name) -> None:
                self.value = name
                super().__init__()

            def on_mount(self) -> None:
                self.update(self.value)

        class Check(Checkbox):
            def __init__(self, value) -> None:
                self.name_value = value
                super().__init__()

            def on_mount(self) -> None:
                self.value = False
                self.id = self.name_value

    BINDINGS = [
        Binding(key="r", action="app.pop_screen", description="return"),
    ]

    async def fetch_date(self) -> None:
        """exams = [(ExamName(value='BUSINESS GAME'), Date(value='08/02/2023'), SigningUp(value='18/01/2023 - 06/02/2023'), Description(value='MDCS 6 ECTS')),
                 (ExamName(value='DATA ANALYTICS'), Date(value='06/02/2023'), SigningUp(value='23/12/2022 - 04/02/2023'), Description(value='Secondo appello')),
                 (ExamName(value='NETWORK SECURITY'), Date(value='07/02/2023'), SigningUp(value='05/01/2023 - 06/02/2023'), Description(value='Oral exam and project discussion')),
                 (ExamName(value='THEORETICAL COMPUTER SCIENCE'), Date(value='28/01/2023'), SigningUp(value='13/01/2023 - 27/01/2023'), Description(value='Prova orale con alcune domande scritte')),
                ]"""
        global wrapper
        try:
            wrapper = cli.new_esse3_wrapper()
        except:
            await self.query_one(".exams-loading").remove()
            await self.query_one(".reservations-container").mount(Static("Login failed !!!", classes="login-failed"))

        exams = wrapper.fetch_exams()
        await self.query_one(".exams-loading").remove()
        if len(exams) == 0:
            await self.query_one(".reservations-container").mount(Static("No exams available !!!", id="exams-empty"))
        else:
            await self.query_one(".reservations-container").mount(
                Vertical(id="exams-table"),
                Static("Select the checkboxes of the exams to be added:", classes="title"),
                Container(classes="reservations-buttons"),
            )
            await self.query_one("#exams-table").mount(self.Tables(exams))
            for index, exam in enumerate(exams, start=1):
                await self.query_one(".reservations-buttons").mount(self.Line(exam))
            await self.query_one(".reservations-container").mount(Horizontal(Button("book", id="add")))


    async def on_mount(self) -> None:
        await asyncio.sleep(0.1)
        asyncio.create_task(self.fetch_date())

    def compose(self) -> ComposeResult:
        yield Header("Exams", classes="header")
        yield Container(Static("List of available exams:", classes="title"),
                        Static("loading [yellow]available exams[/] in progress.....", classes="exams-loading"),
                        classes="reservations-container"
                        )
        yield Footer()

    class Add(Screen):

        def __init__(self, exams) -> None:
            self.exams = exams
            super().__init__()

        async def fetch_date(self) -> None:
            added, click = cli.new_esse3_wrapper().add(self.exams)
            await self.query_one(".exams-loading").remove()
            self.query_one(Container).mount \
                (Static(f"Exams: [green]{', '.join(map(str, added))}[/] added\n"
                        f"\n[bold]click saved: [blue]{click}", id="exams-added-success"))

        async def on_mount(self) -> None:
            await asyncio.sleep(0.1)
            asyncio.create_task(self.fetch_date())

        def compose(self) -> ComposeResult:
            yield Header("Exam added", classes="header")
            yield Container(Static("adding [yellow]reservations[/] in progress.....", classes="exams-loading"))
            yield Footer()

        BINDINGS = [
            Binding(key="r", action="app.return('exams')", description="return"),
            Binding(key="h", action="app.homepage('exams')", description="homepage"),
        ]



import asyncio

from esse3_student import cli

from textual.app import ComposeResult
from textual.binding import Binding
from textual.widgets import Static, Button, Footer, Checkbox
from textual.containers import Container, Horizontal, Vertical
from textual.screen import Screen

from rich import box
from rich.table import Table


class Header(Static):
    pass


class Reservations(Screen):

    class Tables(Static):

        def __init__(self, reservation, index: int) -> None:
            self.reservation = reservation
            self.index = index
            super().__init__()

        def table(self, reservation, index: int):

            table = Table(box=box.SIMPLE_HEAD, style="rgb(139,69,19)")
            table.add_column("#", justify="center", style="bold red")
            for colum in reservation.keys():
                if colum == "Name":
                    table.add_column(colum, justify="center", style="bold cyan", no_wrap=True)
                elif colum == "Date":
                    table.add_column(colum, justify="center", style="bold yellow")
                elif colum == "Cancella Prenotazione":
                    table.add_column(colum, justify="center", style="bold red")
                else:
                    table.add_column(colum, justify="center", style="bold #f7ecb5")

            row = list(reservation.values())
            table.add_row(str(index), *row)
            self.update(table)

        def on_mount(self) -> None:
            self.table(self.reservation, self.index)

    class Line(Horizontal):

        def __init__(self, reservation) -> None:
            self.values = list(reservation.values())
            super().__init__()

        def on_mount(self) -> None:
            self.mount(self.Name(self.values[0]))
            self.mount(self.Check(self.values[0]))

        class Name(Static):
            def __init__(self, name) -> None:
                self.value = name
                super().__init__()

            def on_mount(self) -> None:
                self.update(self.value)

        class Check(Checkbox):
            def __init__(self, value) -> None:
                self.name_value = value
                super().__init__()

            def on_mount(self) -> None:
                self.value = False
                self.id = self.name_value

    BINDINGS = [
        Binding(key="r", action="app.pop_screen", description="return"),
    ]

    async def fetch_date(self) -> None:
        """reservations = [{"name": "TRAINING", "Date": "10/12/2022  09:00", "Appello": "appello sessione straordinaria", "Numero Iscrizione": "15 su 22", "Svolgimento Esame": "Esame in Presenza",
                          "Docenti": "PERRI SIMONA RICCA FRANCESCO", "Cancella Prenotazione": "Impossibile cancellare l'iscrizione: iscrizione chiusa"},
                        {"name": "PROCESS MINING", "Date": "24/01/2023  09:00", "Appello": "First Exam session", "Numero Iscrizione": "19 su 23", "Svolgimento Esame": "Esame in Presenza",
                         "Edificio": "Cubo 30B", "Aula": "MT6", "Docenti": "FIONDA VALERIA", "Cancella Prenotazione": "Impossibile cancellare l'iscrizione: iscrizione chiusa"},
                        {"name": "CYBER OFFENSE AND DEFENSE", "Date": "31/01/2023  09:00", "Appello": "Scritto, discussione ed eventuale orale", "Numero Iscrizione": "1 su 8", "Svolgimento Esame": "Esame in Presenza",
                         "Docenti": "ALVIANO MARIO"},
                       ]"""
        global wrapper
        try:
            wrapper = cli.new_esse3_wrapper()
        except:
            await self.query_one(".reservations-loading").remove()
            await self.query_one(".reservations-container").mount(Static("Login failed !!!", classes="login-failed"))

        reservations = wrapper.fetch_reservations()
        await self.query_one(".reservations-loading").remove()
        if len(reservations) == 0:
            await self.query_one(".reservations-container").mount(Static(f"❌ No appeals booked !!", classes="reservations-empty"))
        else:
            await self.query_one(".reservations-container").mount(
                Vertical(classes="reservations-table"),
                Static("Select the checkboxes of the exams to be removed:", classes="title"),
                Container(classes="reservations-buttons"),
            )
            for index, reservation in enumerate(reservations, start=1):
                self.query_one(Vertical).mount(self.Tables(reservation, index))
                await self.query_one(".reservations-buttons").mount(self.Line(reservation))
            await self.query_one(".reservations-container").mount(Horizontal(Button("remove", id="remove")))

    async def on_mount(self) -> None:
        await asyncio.sleep(0.1)
        asyncio.create_task(self.fetch_date())

    def compose(self) -> ComposeResult:
        yield Header("Reservations", classes="header")
        yield Container(Static("List of Reservations:", classes="title"),
                        Static("loading [yellow]reservations[/] in progress.....", classes="reservations-loading"),
                        classes="reservations-container")
        yield Footer()

    class Remove(Screen):

        def __init__(self, reservations) -> None:
            self.reservations = reservations
            super().__init__()

        async def fetch_date(self) -> None:
            values, click = cli.new_esse3_wrapper().remove(self.reservations)

            all_success = True
            all_closed = True
            for i in values.keys():
                if i == 0:
                    all_success = False
                else:
                    all_closed = False

            await self.query_one("#reservations-loading-removed").remove()

            if all_closed:
                self.query_one(Container).mount(
                    Static(f"❌ Impossible to remove: [red]{', '.join([x for x in values[0]])}[/] cause subscription closed\n"
                           f"\n[bold]click saved: [blue]{click}",
                           classes="reservations-removed-error"))
            elif all_success:
                self.query_one(Container).mount(
                    Static(f"Reservations: [green]{', '.join([x for x in values[1]])}[/] removed\n"
                           f"\n[bold]click saved: [blue]{click}",
                           id="reservations-removed-success"))
            else:
                self.query_one(Container).mount(
                    Static(f"✅ Reservations: [green]{', '.join([x for x in values[1]])}[/] removed \n\n"
                           f"❌ Impossible to remove: [red]{', '.join([x for x in values[0]])}[/] cause subscription closed\n"
                           f"\n[bold]click saved: [blue]{click}",
                           classes="reservations-removed-mix"))

        async def on_mount(self) -> None:
            await asyncio.sleep(0.1)
            asyncio.create_task(self.fetch_date())

        def compose(self) -> ComposeResult:
            yield Header("Reservations removed page", classes="header")
            yield Container(Static("Reservations [yellow]removal[/] in progress.....", id="reservations-loading-removed"))
            yield Footer()

        BINDINGS = [
            Binding(key="r", action="app.return('exams')", description="return"),
            Binding(key="h", action="app.homepage('reservations')", description="homepage"),
        ]


import asyncio

from esse3_student.primitives import Cfu, Grade
from esse3_student import cli

from textual.app import ComposeResult
from textual.binding import Binding
from textual.widgets import Static, Button, Footer, Input
from textual.containers import Container, Vertical
from textual.screen import Screen

from rich import box
from rich.text import Text
from rich.table import Table


class Header(Static):
    pass


def get_status(status: str) -> Text:
    colors = {"Passed": "green", "Ex officio assigned frequency": "#f7ecb5"}
    value = Text(status)
    value.stylize(f"{colors[status]}")

    return value


def get_grade(grade: str) -> Text:
    colors = {"18": "bright_red", "19": "bright_red", "20": "bright_red", "21": "bright_red",
              "22": "yellow", "23": "yellow", "24": "yellow", "25": "yellow",
              "26": "blue", "27": "blue", "28": "blue", "29": "blue", "30": "blue",
              '': "white", "...": "white", "ELIGIBLE": "green"}
    value = Text(grade)
    value.stylize(f"{colors[grade]}")

    return value


class Booklet(Screen):

    exams = None
    statistics = None

    class Tables(Static):

        def __init__(self, exams) -> None:
            self.exams = exams
            super().__init__()

        def on_mount(self):
            self.id = "booklet-table-exams"
            self.table(self.exams)

        def table(self, exams) -> None:

            table = Table(style="rgb(139,69,19) bold", box=box.SIMPLE_HEAD)
            table.add_column("#", style="red bold")
            table.add_column("Name", style="cyan bold")
            table.add_column("Academic Year", style="bold", justify="center")
            table.add_column("CFU", style="bold", justify="center")
            table.add_column("Status", style="bold")
            table.add_column("Grade", style="bold", justify="center")
            table.add_column("Date", style="#E1C699 bold", justify="center")

            for index, (name, year, cfu, status, grade, date) in enumerate(exams, start=1):
                name, year, cfu, status, grade, date = map(lambda x: x.value, (name, year, cfu, status, grade, date))

                status = get_status(status)
                grade = get_grade(grade)

                table.add_row(str(index), name, str(year), cfu, status, grade, date)

            self.update(table)

    class ActualAverage(Static):

        def __init__(self, averages) -> None:
            self.averages = averages
            super().__init__()

        def on_mount(self):
            self.table(self.averages)

        def table(self, averages):
            table = Table(header_style="rgb(255,204,51) bold", box=box.SIMPLE_HEAD)
            table.add_column("Actual Average", style="bold", justify="center")
            table.add_column("Actual Degree basis", style="bold", justify="center")

            actual_average, actual_cfu  = averages
            degree_basis = (float(actual_average) * 11) / 3

            table.add_row(str(actual_average), str(round(degree_basis, 2)))
            self.update(table)

    class NewAverage(Static):

        def __init__(self, averages) -> None:
            self.averages = averages
            super().__init__()

        def compose(self) -> ComposeResult:
            yield Container(
                Static("[b]What average would you like to have?[/]"),
                Input(placeholder="average..", id="average"),
                Static("[b]How many cfu do you still have available?[/]"),
                Input(placeholder="cfu...", id="cfu"),
                Button("[b]compute[/]", classes="compute", id="compute-average"),
                classes="booklet-container-filters",
            )

        def on_button_pressed(self, event: Button.Pressed):

            if event.button.id == "compute-average":
                buttons = ["#result", "#booklet-value-error"]
                for element in buttons:
                    try:
                        self.query(element).last().remove()
                    except:
                        pass
                average_to_achieve = self.query_one("#average").value
                remaining_cfu = self.query_one("#cfu").value
                if average_to_achieve != "" and remaining_cfu != "":
                    try:

                        average_to_achieve = Grade(int(average_to_achieve)).value
                        remaining_cfu = int(remaining_cfu)

                        actual_average, actual_cfu = self.averages

                        total_cfu = remaining_cfu + actual_cfu

                        grade_to_obtain = ((average_to_achieve * total_cfu) - (actual_average * actual_cfu)) \
                                          / float(remaining_cfu)

                        self.query_one(".booklet-container-filters").mount(Static(f"[b]The grade you need to take \n"
                                                                                  f"for each upcoming exam\n"
                                                                                  f"having [yellow]{remaining_cfu} cfu available[/]: "
                                                                                  f"[green]{str(round(grade_to_obtain, 2))}", id="result"))

                    except ValueError:
                        self.query_one(".booklet-container-filters").mount(
                            Static("[red][bold]Wrong values[/]", id="booklet-value-error"))

    class NewGrade(Static):

        def __init__(self, averages) -> None:
            self.averages = averages
            super().__init__()

        def compose(self) -> ComposeResult:
            yield Container(
                Static("[b]What grade do you think you get?[/]"),
                Input(placeholder="grade...", id="grade"),
                Static("[b]How many CFU does the exam have?[/]"),
                Input(placeholder="cfu...", id="cfu"),
                Button("[b]compute[/]", classes="compute", id="compute-grade"),
                classes="booklet-container-filters",
            )

        def on_button_pressed(self, event: Button.Pressed):

            if event.button.id == "compute-grade":
                buttons = ["#result", "#booklet-value-error"]
                for element in buttons:
                    try:
                        self.query(element).last().remove()
                    except:
                        pass
                grade = self.query_one("#grade").value
                cfu = self.query_one("#cfu").value
                if grade != "" and cfu != "":
                    try:
                        grade = Grade(int(grade)).value
                        cfu = int(Cfu(cfu).value)
                        actual_average, actual_cfu = self.averages
                        new_average = ((actual_average * actual_cfu) + (int(grade) * int(cfu))) / (
                                actual_cfu + int(cfu))
                        new_degree_basis = (new_average * 11.0) / 3.0

                        self.query_one(".booklet-container-filters").mount(Static(
                            f"[b][yellow]New Average:[/] {round(new_average, 2)}/30\n[yellow]New Degree basis:[/] "
                            f"{round(new_degree_basis, 2)}/110[/]",
                            id="result"))

                    except ValueError:
                        self.query_one(".booklet-container-filters").mount(
                            Static("[red][bold]Wrong values[/]", id="booklet-value-error"))

    async def fetch_data(self) -> None:

        global wrapper
        try:
            wrapper = cli.new_esse3_wrapper()
        except:
            await self.query_one("#booklet-loading").remove()
            await self.query_one("#booklet-container").mount(Static("Login failed !!!", classes="login-failed"))

        self.exams, self.statistics = wrapper.fetch_booklet()

        await self.query_one("#booklet-loading").remove()
        await self.query_one("#booklet-container").mount(
            Vertical(self.Tables(self.exams))
        )
        await self.query_one("#booklet-container").mount(
                            Static("Planning:", classes="title"),
                            Container(
                                self.ActualAverage(self.statistics),
                                Button("Plan a new average", id="average"),
                                Button("Projects a new grade", id="grade"),
                                Button("clear", id="clear"),
                                classes="booklet-container-options"
                            ),
                        )

    async def on_mount(self) -> None:
        await asyncio.sleep(0.1)
        asyncio.create_task(self.fetch_data())

    def compose(self) -> ComposeResult:
        yield Header("Booklet", classes="header")
        yield Container(
            Static("Exams:", classes="title"),
            Static("[b][yellow]exams booklet[/] loading in progress....[/]", id="booklet-loading"),
            id="booklet-container"
        )
        yield Footer()

    def on_button_pressed(self, event: Button.Pressed):

        elements_to_remove = [self.NewAverage, self.NewGrade]
        compute_buttons = ["compute-average", "compute-grade"]

        if event.button.id == "clear":
            for element in elements_to_remove:
                try:
                    self.query(element).last().remove()
                except:
                    pass
            return

        if event.button.id not in compute_buttons:
            for element in elements_to_remove:
                try:
                    self.query(element).last().remove()
                except:
                    pass

        if event.button.id == "average":
            self.query_one("#booklet-container").mount(self.NewAverage(self.statistics))

        if event.button.id == "grade":
            self.query_one("#booklet-container").mount(self.NewGrade(self.statistics))

        if event.button.id == "degree":
            self.query_one("#booklet-container").mount(self.NewDegree(self.exams))

    BINDINGS = [
        Binding(key="r", action="app.pop_screen", description="return"),
    ]


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['esse3_student', 'esse3_student.tui', 'esse3_student.utils']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.3,<4.0.0',
 'asyncselenium>=0.0.2,<0.0.3',
 'dataclass-type-validator>=0.1.2,<0.2.0',
 'dateutils>=0.6.12,<0.7.0',
 'export>=0.2.0,<0.3.0',
 'httpx>=0.23.3,<0.24.0',
 'pytest-localserver>=0.7.0,<0.8.0',
 'python-dotenv>=0.21.1,<0.22.0',
 'rich>=12.5.1,<13.0.0',
 'screeninfo>=0.8.1,<0.9.0',
 'selenium>=4.4.3,<5.0.0',
 'self>=2020.12.3,<2021.0.0',
 'termcolor>=2.2.0,<3.0.0',
 'textual>=0.6.0,<0.7.0',
 'typeguard>=2.13.3,<3.0.0',
 'typer>=0.6.1,<0.7.0',
 'valid8>=5.1.2,<6.0.0',
 'webdriver-manager>=3.8.3,<4.0.0']

extras_require = \
{':python_version >= "3.10" and python_version < "4.0"': ['xdotool>=0.4.0,<0.5.0']}

setup_kwargs = {
    'name': 'esse3-student',
    'version': '3.0.0',
    'description': 'Esse3 command line utility',
    'long_description': 'None',
    'author': 'Antonio Pallaria',
    'author_email': 'pallaria98@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import asyncio

from esse3_student import cli
from typing import Any

from textual.app import ComposeResult
from textual.binding import Binding
from textual.widgets import Static, Footer
from textual.containers import Container, Vertical
from textual.screen import Screen

from rich import box
from rich.text import Text
from rich.table import Table


class Header(Static):
    pass


class Taxes(Screen):

    class Tables(Static):

        def __init__(self, taxes) -> None:
            self.taxes = taxes
            super().__init__()

        def on_mount(self) -> None:
            self.id = "taxes-table"
            self.table(self.taxes)

        def payment_changes(self, amount: str, payment_status) -> tuple[Text, Any]:
            colors = {" pagato confermato": "rgb(50,205,50)", " non pagato": "red", " pagato": "rgb(0,100,0)"}
            names = {" pagato confermato": "payment confirmed", " non pagato": "to pay", " pagato": "refund"}
            value = Text(amount)
            value.stylize(f"{colors[payment_status]}")
            return value, names[payment_status]

        def table(self, taxes) -> None:

            table = Table(style="rgb(139,69,19) bold", box=box.SIMPLE_HEAD)
            table.add_column("#", style="red bold")
            table.add_column("ID", style="cyan bold")
            table.add_column("Expiration date", style="bold yellow", justify="center")
            table.add_column("Amount", style="bold")
            table.add_column("Payment status", style="bold #f7ecb5")

            for index, (id, date, amount, status) in enumerate(taxes, start=1):
                id, date, amount, status = map(lambda x: x.value, (id, date, amount, status))
                amount, payment = self.payment_changes(amount, status)
                table.add_row(str(index), id, date, amount, payment)

            self.update(table)

    async def fetch_date(self) -> None:

        global wrapper
        try:
            wrapper = cli.new_esse3_wrapper()
        except:
            await self.query_one("#taxes-loading").remove()
            await self.query_one(Container).mount(Static("Login failed !!!", classes="login-failed"))

        taxes, statistics = wrapper.fetch_taxes()
        await self.query_one("#taxes-loading").remove()
        self.query_one(Container).mount(
            Vertical(
                self.Tables(taxes),
            )
        )

    async def on_mount(self) -> None:
        await asyncio.sleep(0.1)
        asyncio.create_task(self.fetch_date())

    def compose(self) -> ComposeResult:
        yield Header("Taxes", classes="header")
        yield Container(
            Static("List of taxes:", classes="title"),
            Static("[yellow]taxes loading[/] in progress.....", id="taxes-loading"),
            id="taxes-container"
        )
        yield Footer()

    BINDINGS = [
        Binding(key="r", action="app.pop_screen", description="return"),
    ]


import dataclasses
import time
import typer

from typing import Tuple
from rich import box
from rich.progress import track
from rich.table import Table
from rich.text import Text

from esse3_student.esse3_wrapper import Esse3Wrapper
from esse3_student.primitives import AcademicYear, ExamStatus, Cfu, Year, ExamName, Grade
from esse3_student.utils.console import console

from typing import Optional


@dataclasses.dataclass(frozen=True)
class AppOptions:
    username: str = dataclasses.field(default='')
    password: str = dataclasses.field(default='')
    debug: bool = dataclasses.field(default=False)


app_options = AppOptions()
app = typer.Typer(rich_markup_mode="rich", add_completion=False)


def is_debug_on():
    return app_options.debug


def run_app():
    try:
        app()
    except Exception as e:
        if is_debug_on():
            raise e
        else:
            console.print(f"[red bold]Error:[/red bold] {e}")


def new_esse3_wrapper(detached: bool = False, with_live_status: bool = True):
    def res():
        return Esse3Wrapper.create(
            username=app_options.username,
            password=app_options.password,
            debug=app_options.debug,
            detached=detached,
            headless=not app_options.debug and not detached,
        )
    if with_live_status:
        with console.status("[bold]Login....[/]"):
            return res()
    return res()


@app.callback()
def main(
        username: str = typer.Option(..., prompt=True, envvar="CLI_STUDENT_USERNAME"),
        password: str = typer.Option(..., "--password", prompt=True, hide_input=True, envvar="CLI_STUDENT_PASSWORD"),
        debug: bool = typer.Option(False, "--debug", help="To show browser operations"),
):

    """
    [bold][green]Esse3 command line utility[/green][/bold] :computer:
    """

    global app_options
    app_options = AppOptions(
        username=username,
        password=password,
        debug=debug,
    )


@app.command(name="exams")
def command_exams() -> None:

    """
    [bold][#E1C699]Show available exams list[/][/bold] :bookmark_tabs:
    """

    esse_wrapper = new_esse3_wrapper()
    with console.status("[bold]Retrieval of [green]available exams[/green] in progress[/]",
                        spinner="aesthetic"):
        time.sleep(2)
        exams = esse_wrapper.fetch_exams()

    if len(exams) == 0:
        console.print("No exams available!!!", style="bold red")
        exit()

    console.rule("[bold]EXAMS SHOWCASE[/]")

    table = Table(box=box.SIMPLE_HEAD, style="rgb(139,69,19)", leading=1)
    table.add_column("#", justify="center", style="bold red")
    table.add_column("Name", justify="center", style="bold cyan")
    table.add_column("Date", justify="center", style="bold green")
    table.add_column("Signing up", justify="center", style="bold yellow")
    table.add_column("Description", justify="center", style="bold #f7ecb5")

    for index, (name, date, signing_up, description) in enumerate(exams, start=1):
        table.add_row(str(index), name.value, date.value, signing_up.value, description.value)

    console.print(table, justify="center")
    console.rule("[bold]STATISTICS[/]", style="yellow")
    console.print("\n[bold]clicks saved: [blue]7[/]\n", justify="center")


@app.command(name="reservations")
def command_reservations() -> None:

    """
    [bold][#E1C699]Show exams booked list[/][/bold] :bookmark_tabs:
    """

    esse_wrapper = new_esse3_wrapper()
    with console.status("[bold]Fetching [green]available reservations[/] in progress....[/]", spinner="aesthetic"):
        time.sleep(2)
        reservations = esse_wrapper.fetch_reservations()
        if len(reservations) == 0:
            console.print("No exams booked!!!", style="bold red")
            exit()

    console.rule("[bold]RESERVATIONS SHOWCASE[/]")
    tables = {}
    for index in range(len(reservations)):
        tables[f'table_{index}'] = Table(box=box.SIMPLE_HEAD, style="rgb(139,69,19)")
        tables[f'table_{index}'].add_column("#", justify="center", style="bold red")
        for colum in reservations[index].keys():
            if colum == "Name":
                tables[f'table_{index}'].add_column(colum, justify="center", style="bold cyan", no_wrap=True)
            elif colum == "Date":
                tables[f'table_{index}'].add_column(colum, justify="center", style="bold yellow")
            elif colum == "Cancella Prenotazione":
                tables[f'table_{index}'].add_column(colum, justify="center", style="bold red")
            else:
                tables[f'table_{index}'].add_column(colum, justify="center", style="bold #f7ecb5")

    for index, reservation in enumerate(reservations, start=0):
        row = list(reservation.values())
        tables[f'table_{index}'].add_row(str(index+1), *row)
        console.print(tables[f'table_{index}'], justify="center")

    console.rule("[bold]STATISTICS[/]", style="yellow")
    console.print("\n[bold]clicks saved: [blue]7[/]\n", justify="center")


@app.command(name="add")
def command_add(
        exams: list[str] = typer.Argument(
            ...,
            metavar="exam names",
            help="[bold]one or more strings of the form 'add name1 name2ssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss'"
        ),
):
    """
    [bold][#E1C699]Operation that allows the [green]booking[/green] of examinations[/][/bold] :blue_book:
    """

    def parse(exams: list) -> list[ExamName]:
        try:
            exams_list = [ExamName(v) for v in exams]
        except ValueError:
            console.print("[bold red]Invalid strings[/]")
            raise typer.Exit()

        return exams_list

    values = parse(exams)

    esse_wrapper = new_esse3_wrapper()

    with console.status(f"[bold]Exams [green]booking[/] in progress....[/]", spinner="aesthetic"):
        exams, click = esse_wrapper.add(list(values))
        values = [i.value for i in exams]
        if len(values) == 0:
            console.print("No exams available or wrong names passed!!!\n", style="bold red")
        else:
            console.log(f"[bold] ✅ Exams with name: [green]{', '.join(map(str, values))}[/] added\n")

    console.rule("[bold]STATISTICS[/]", style="yellow")
    console.print(f"\n[bold]clicks saved: [blue]{click}[/]\n", justify="center")


@app.command(name="remove")
def command_remove_reservation(
        reservations: list[str] = typer.Argument(
            ...,
            metavar="Reservations name",
            help="[bold]one or more strings of the form 'remove name1 name2'"
        ),

):

    """
    [bold][#E1C699]Operation that allows the [red]deletion[/red] of booked examinations[/][/bold] :wastebasket:
    """

    def parse(reservations: list) -> list[ExamName]:
        try:
            exams_list = [ExamName(v) for v in reservations]
        except ValueError:
            console.print("[bold red]Invalid strings[/]")
            raise typer.Exit()

        return exams_list

    values = parse(reservations)

    esse3_wrapper = new_esse3_wrapper()

    with console.status(f"[bold]Searching [green]reservations[/] to remove in progress....[/]", spinner="aesthetic"):
        time.sleep(2)
        values, click = esse3_wrapper.remove(list(values))

        if len(values) == 0:
            console.log(f"[bold]❌ No exams to remove or wrong values passed[/]!!!")
        else:
            all_success = True
            all_closed = True
            for i in values.keys():
                if i == 0:
                    all_success = False
                else:
                    all_closed = False

            if all_closed:
                console.log(
                    f"[bold]❌ Impossible to remove: [red]{', '.join([x for x in values[0]])}[/] cause subscription closed or wrong names passed[/]")
            elif all_success:
                console.log(f"[bold]Reservations: [green]{', '.join([x for x in values[1]])}[/] removed\n[/]")
            else:
                console.log(f"[bold]✅ Reservations: [green]{', '.join([x for x in values[1]])}[/] removed[/]")
                console.log(
                    f"[bold]❌ Impossible to remove: [red]{', '.join([x for x in values[0]])}[/] cause subscription closed or wrong names passed[/]")

    console.rule("[bold]STATISTICS[/]", style="yellow")
    console.print(f"\n[bold]clicks saved: [blue]{click}[/]\n", justify="center")


@app.command(name="booklet")
def command_booklet(
        academic_year: int = typer.Option(int, help="[bold]Academic year (1 to 3)"),
        exam_status: str = typer.Option(..., help="[bold]'[green]Superata[/]' like 'Passed' or '[yellow]Frequenza attribuita d'ufficio[/]' like 'To do'[/]"),
        exam_grade: int = typer.Option(int, help="[bold]Grade of the exam[/]"),
        new_average: Tuple[int, str] = typer.Option((None, None), help="[bold]calculate new average with grade: (grade cfu); ex: '25 12' [/]"),
        #new_degree: Tuple[int, str] = typer.Option((None, None), help="[bold]calculate new average with remaining cfu: (degree cfu); ex: '27 12' [/]"),
        statistics: Optional[bool] = typer.Option(False, "--statistics", "-s", help="[bold]show statistics on the average"),
        #prova: int = typer.Argument(int, help="caaooooooooooooooooooooo"), #valore predefinito è zero in questo caso
) -> None:

    """
    [bold][#E1C699]shows all the student's activities[/][/bold] :bookmark_tabs:
    """

    if academic_year:
        try:
            academic_year = AcademicYear(academic_year)
        except ValueError:
            console.print("[bold yellow]Invalid year[/]")
            raise typer.Exit()

    if exam_status:
        try:
            exam_status = ExamStatus(exam_status)
        except ValueError:
            console.print("[bold yellow]Invalid exam status[/]")
            raise typer.Exit()

    if exam_grade:
        try:
            exam_grade = Grade(exam_grade)
        except ValueError:
            console.print("[bold yellow]Invalid grade[/]")
            raise typer.Exit()

    new_vote = None
    new_cfu = None
    if new_average[1]:
        try:
            new_vote = Grade(int(new_average[0])).value
        except ValueError:
            console.print("[bold yellow]Invalid vote value[/]")
            raise typer.Exit()
        try:
            new_cfu = int(Cfu(new_average[1]).value)
        except ValueError:
            console.print("[bold yellow]Invalid cfu value[/]")
            raise typer.Exit()

    esse3_wrapper = new_esse3_wrapper()
    with console.status("[bold]Fetching [green]exams booklet[/] in progress....[/]", spinner="aesthetic"):
        exams, averages = esse3_wrapper.fetch_booklet()

    table = Table(style="rgb(139,69,19) bold", box=box.SIMPLE_HEAD)
    table.add_column("#", style="red bold")
    table.add_column("Name", style="cyan bold")
    table.add_column("Academic Year", style="bold", justify="center")
    table.add_column("CFU", style="bold", justify="center")
    table.add_column("Status", style="bold")
    table.add_column("Grade", style="bold", justify="center")
    table.add_column("Date", style="#E1C699 bold", justify="center")

    def get_status_color(status):
        colors = {"Passed": "green", "Ex officio assigned frequency": "yellow"}
        return colors[status]

    def get_grade_color(grade):
        colors = {"18": "bright_red", "19": "bright_red", "20": "bright_red", "21": "bright_red",
                  "22": "yellow", "23": "yellow", "24": "yellow", "25": "yellow",
                  "26": "blue", "27": "blue", "28": "blue", "29": "blue", "30": "blue",
                  '': "white", "...": "white", "ELIGIBLE": "green"}
        return colors[grade]

    for index, (name, year, cfu, status, grade, date) in enumerate(exams, start=1):

        name, year, cfu, status, grade, date = map(lambda x: x.value, (name, year, cfu, status, grade, date))

        status_color = get_status_color(status)
        grade_color = get_grade_color(grade)

        grade_style = Text(grade)
        grade_style.stylize(f"{grade_color}")

        if academic_year and year != academic_year.value:
            continue
        if exam_status and status != exam_status.value:
            continue
        if exam_grade and grade[0:2] != str(exam_grade.value):
            continue

        table.add_row(str(index), name, str(year), cfu, f'[{status_color}]{status}[/{status_color}]',
                      grade_style, date)

    console.rule("[bold]BOOKLET SHOWCASE[/bold]")
    console.print(table, justify="center")

    table = Table(header_style="rgb(210,105,30) bold", box=box.SIMPLE_HEAD)
    table.add_column("Weighted average", style="bold", justify="center")
    table.add_column("Degree basis", style="bold", justify="center")

    actual_average, actual_cfu = averages
    degree_basis = (actual_average * 11) / 3

    if new_average[1] is not None:
        table.add_column("Grade", style="bold", justify="center")
        table.add_column("Cfu", style="bold", justify="center")
        table.add_column("New Average", style="bold", justify="center")
        table.add_column("New Degree Basis", style="bold", justify="center")

        new = ((actual_average * actual_cfu) + (new_vote * new_cfu)) / (actual_cfu + new_cfu)
        new_degree_basis = (new * 11) / 3

        table.add_row(str(actual_average), str(round(degree_basis, 2)), str(new_vote),
                      str(new_cfu), str(round(new, 2)), str(round(new_degree_basis, 2)))
    else:
        table.add_row(str(actual_average), str(round(degree_basis, 2)))

    if statistics:
        console.print(table, justify="center")
    console.rule("[bold]STATISTICS[/bold]")
    console.print("\n[bold]clicks saved: [blue]7[/]\n", justify="center")


@app.command(name="taxes")
def command_taxes(
        payment: str = typer.Option(str, help="[bold]Shows taxes by status type: ('to pay'|'confirmed'|'refund')"),
        year: int = typer.Option(int, "--year", "-y", help="[bold]filter taxes by year; es: '2021'"),
) -> None:

    """
    [bold][#E1C699]Show all taxes[/][/bold] :bookmark_tabs:
    """

    if year:
        try:
            year = Year(year).value
        except ValueError:
            console.print("Invalid year value")
            raise typer.Exit()

    esse3_wrapper = new_esse3_wrapper()
    with console.status("[bold]Fetching [green]taxes[/] in progress....[/]", spinner="aesthetic"):
        taxes, click = esse3_wrapper.fetch_taxes()

    table = Table(style="rgb(139,69,19) bold", box=box.SIMPLE_HEAD)
    table.add_column("#", style="red bold")
    table.add_column("ID", style="cyan bold")
    table.add_column("Expiration date", style="bold yellow", justify="center")
    table.add_column("Amount", style="bold")
    table.add_column("Payment status", style="bold #f7ecb5")

    def payment_changes(payment_status) -> Tuple[str, str]:
        colors = {" pagato confermato": "rgb(50,205,50)", " non pagato": "red", " pagato": "rgb(0,100,0)"}
        names = {" pagato confermato": "payment confirmed", " non pagato": "to pay", " pagato": "refund"}
        return names[payment_status], colors[payment_status]

    for index, (id, date, amount, status) in enumerate(track(taxes, description="[bold]Processing....[/]", transient=True), start=1):
        id, date, amount, status = map(lambda x: x.value, (id, date, amount, status))
        payment_status, c = payment_changes(status)

        if payment in payment_status and str(year) in date:
            table.add_row(str(index), id, date, f'[{c}]{amount}[/{c}]', payment_status)

        elif payment:
            if payment in payment_status:
                table.add_row(str(index), id, date, f'[{c}]{amount}[/{c}]', payment_status)

        elif year:
            if str(year) in date:
                table.add_row(str(index), id, date, f'[{c}]{amount}[/{c}]', payment_status)

        else:
            table.add_row(str(index), id, date, f'[{c}]{amount}[/{c}]', payment_status)

    console.rule("[bold]TAXES SHOWCASE[/bold]")
    console.print(table, justify="center")
    console.rule("[bold]STATISTICS[/]", style="yellow")
    console.print(f"\n[bold]clicks saved: [blue]{click}[/]\n", justify="center")


@app.command(name="tui")
def tui() -> None:

    """
    [bold][#E1C699]Run [yellow]text-user-interface[/yellow][/][/bold]
    """
    from esse3_student.tui.main import Tui

    Tui().run()


